//
//  MBCoreDataReorderableTableViewController.m
//
//  Created by Martina Brazelton on 6/8/13.
//  Copyright (c) 2013 Martina Brazelton. All rights reserved.
//

#import "MBCoreDataReorderableTableViewController.h"
#import "MBReorderableTableView.h"


@implementation MBCoreDataReorderableTableViewController

- (id)initWithFetchedResultsController:(NSFetchedResultsController *)fetchedResultsController
{
    self = [super initWithFetchedResultsController:fetchedResultsController];
    if (self) {
        [self initialize];
    }
    
    return self;
}

- (void)initialize
{

}

- (MBReorderableTableView *)reorderableTableView
{
    MBReorderableTableView *reorderableTableview = [MBReorderableTableView.alloc initWithFrame:self.view.frame style:UITableViewStylePlain];
    
    [reorderableTableview setStartReorderHandler:^(MBReorderableTableView *tableview, NSIndexPath *oldIndexPath){
        id object = [self moveStartedAtIndexPath:oldIndexPath];
        return object;
    }];
    
    [reorderableTableview setReorderHandler:^(MBReorderableTableView *tableview, NSIndexPath *oldIndexPath, NSIndexPath *indexPath){
        [self moveInProgress:oldIndexPath newIndexPath:indexPath];
    }];
    
    [reorderableTableview setCompletionHandler:^(MBReorderableTableView *tableview, NSIndexPath *oldIndexPath, NSIndexPath *indexPath, id object){
        [self object:object movedToIndexPath:indexPath];
    }];
    
    return reorderableTableview;
}

- (id)moveStartedAtIndexPath:(NSIndexPath *)indexPath
{
    NSManagedObject *managedObject = [self managedObjectAtIndexPath:indexPath];
    [self replaceObjectAtIndexPath:indexPath withObject:self.dummyObject];
    
    return managedObject;
}

- (void)moveInProgress:(NSIndexPath *)oldIndexPath newIndexPath:(NSIndexPath *)indexPath
{
    NSManagedObject *managedObject = [self managedObjectAtIndexPath:oldIndexPath];
    [self removeObjectAtIndexPath:oldIndexPath andInsertObject:managedObject atIndexPath:indexPath];
}

- (void)object:(id)object movedToIndexPath:(NSIndexPath *)indexPath
{
    [self replaceObjectAtIndexPath:indexPath withObject:object];
    [self saveNewOrder];
}

- (void)saveNewOrder
{
    // intended to be overridden
}

- (void)replaceObjectAtIndexPath:(NSIndexPath *)indexPath withObject:(id)object
{
    [self.managedObjects replaceObjectAtIndex:indexPath.row withObject:object];
}

- (void)removeObjectAtIndexPath:(NSIndexPath *)removeIndexPath andInsertObject:(id)object atIndexPath:(NSIndexPath *)insertIndexPath
{
    [self.managedObjects removeObjectAtIndex:removeIndexPath.row];
    [self.managedObjects insertObject:object atIndex:insertIndexPath.row];
}

- (id)objectAtIndexPath:(NSIndexPath *)indexPath
{
    return [self.managedObjects objectAtIndex:indexPath.row];
}

- (id)dummyObject
{
    return @"DUMMY";
}

- (void)configureDefaultSettingsForCell:(UITableViewCell *)cell atIndexPath:(NSIndexPath *)indexPath
{
    cell.contentView.backgroundColor = UIColor.clearColor;
    cell.accessoryType = UITableViewCellAccessoryNone;
}

- (void)configureDummyCell:(UITableViewCell *)cell forIndexPath:(NSIndexPath *)indexPath
{
    cell.textLabel.text = @"";
    cell.detailTextLabel.text = @"";
    cell.imageView.hidden = YES;
    cell.accessoryType = UITableViewCellAccessoryNone;
}

- (void)configureCell:(UITableViewCell *)cell forIndexPath:(NSIndexPath *)indexPath
{    

}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
    UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:CELL_REUSE_ID forIndexPath:indexPath];
    
    [self configureDefaultSettingsForCell:cell atIndexPath:indexPath];
    
    if ([[self objectAtIndexPath:indexPath] isEqual:self.dummyObject]) {
        [self configureDummyCell:cell forIndexPath:indexPath];
    } else {
        [self configureCell:cell forIndexPath:indexPath];
    }
    
    return cell;
}

- (BOOL)performFetch
{
    BOOL success = [super performFetch];
    self.managedObjects = self.fetchedResultsController.fetchedObjects.mutableCopy;
    
    return success;
}

- (void)viewDidLoad
{
    [super viewDidLoad];
    self.tableView = self.reorderableTableView;
    [self.tableView registerClass:self.cellClass forCellReuseIdentifier:CELL_REUSE_ID];
}

@end
