//
//  MBCoreDataTableViewController.h
//
//  Created by Martina Brazelton on 6/1/13.
//  Copyright (c) 2013 Martina Brazelton. All rights reserved.
//

extern NSString *const CELL_REUSE_ID;
extern NSString *const HEADER_REUSE_ID;

@interface MBCoreDataTableViewController : UITableViewController <NSFetchedResultsControllerDelegate>

@property (nonatomic, strong, readonly) NSFetchedResultsController *fetchedResultsController;
@property (nonatomic, assign) NSManagedObjectContext *context;

+ (MBCoreDataTableViewController *)coreDataTableViewControllerWithFetchedResultsController:(NSFetchedResultsController *)fetchedResultsController;
- (Class)cellClass;
- (Class)headerClass;
- (NSString *)cellReuseID;
- (NSString *)headerReuseID;
+ (NSFetchedResultsController *)fetchedResultsControllerWithFetchRequest:(NSFetchRequest *)fetchRequest context:(NSManagedObjectContext *)context sectionNameKeyPath:(NSString *)sectionNameKeyPath cacheName:(NSString *)cacheName delegate:(id<NSFetchedResultsControllerDelegate>)delegate;
+ (NSFetchRequest *)fetchRequestWithEntity:(NSString *)entityName sortDescriptors:(NSArray *)sortDescriptors predicate:(NSPredicate *)predicate;
- (id)initWithFetchedResultsController:(NSFetchedResultsController *)fetchedResultsController;
- (BOOL)performFetch;
- (void)handleFetchError:(NSError *)error;
- (void)handleSaveError:(NSError *)error;
- (BOOL)saveChanges;
- (id <NSFetchedResultsSectionInfo>)sectionAtIndex:(NSInteger)section;
- (NSManagedObject *)managedObjectAtIndexPath:(NSIndexPath *)indexPath;
- (NSIndexPath *)indexPathForManagedObject:(NSManagedObject *)managedObject;
- (NSManagedObject *)managedObjectAtCell:(UITableViewCell *)cell;
- (UITableViewCell *)cellForManagedObject:(NSManagedObject *)managedObject;
- (void)configureCell:(UITableViewCell *)cell forIndexPath:(NSIndexPath *)indexPath;
- (void)configureHeader:(UIView *)header forSection:(NSInteger)section;
- (void)didSelectManagedObject:(NSManagedObject *)managedObject;

@end
