//
//  MBReorderableTableViewController.m
//  ListedApp
//
//  Created by Martina Brazelton on 6/8/13.
//  Copyright (c) 2013 Martina Brazelton. All rights reserved.
//

#import "MBReorderableTableViewController.h"
#import "MBReorderableTableView.h"

static NSString *CellIdentifier = @"CellID";
static NSString *HeaderIdentifier = @"HeaderID";

@interface MBReorderableTableViewController ()

@end

@implementation MBReorderableTableViewController

- (id)initWithStyle:(UITableViewStyle)style
{
    self = [super initWithStyle:style];
    if (self) {
        [self setup];
    }
    
    return self;
}

- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil
{
    self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil];
    if (self) {
        [self setup];
    }
    
    return self;
}

- (id)initWithCoder:(NSCoder *)aDecoder
{
    self = [super initWithCoder:aDecoder];
    if (self) {
        [self setup];
    }
    
    return self;
}

- (id)init
{
    self = super.init;
    if (self) {
        [self setup];
    }
    
    return self;
}

- (void)setup
{
    self.objects = NSMutableArray.new;
}

- (Class)cellClass
{
    return UITableViewCell.class;
}

- (NSString *)cellReuseID
{
    return CellIdentifier;
}

- (Class)headerClass
{
    return nil;
}

- (NSString *)headerReuseID
{
    return HeaderIdentifier;
}

- (MBReorderableTableView *)reorderableTableView
{
    MBReorderableTableView *reorderableTableview = [MBReorderableTableView.alloc initWithFrame:self.view.frame style:UITableViewStylePlain];
    
    [reorderableTableview setStartReorderHandler:^(MBReorderableTableView *tableview, NSIndexPath *oldIndexPath){        
        id object = [self moveStartedAtIndexPath:oldIndexPath];
        return object;
    }];
    
    [reorderableTableview setReorderHandler:^(MBReorderableTableView *tableview, NSIndexPath *oldIndexPath, NSIndexPath *indexPath){
        [self moveInProgress:oldIndexPath newIndexPath:indexPath];
    }];
    
    [reorderableTableview setCompletionHandler:^(MBReorderableTableView *tableview, NSIndexPath *oldIndexPath, NSIndexPath *indexPath, id object){
        [self object:object movedToIndexPath:indexPath];
    }];
    
    return reorderableTableview;
}

- (id)moveStartedAtIndexPath:(NSIndexPath *)indexPath
{
    id object = [self objectAtIndexPath:indexPath];
    [self replaceObjectAtIndexPath:indexPath withObject:self.dummyObject];
    
    return object;
}

- (void)moveInProgress:(NSIndexPath *)oldIndexPath newIndexPath:(NSIndexPath *)indexPath
{
    id object = [self objectAtIndexPath:oldIndexPath];
    [self removeObjectAtIndexPath:oldIndexPath andInsertObject:object atIndexPath:indexPath];
}

- (void)object:(id)object movedToIndexPath:(NSIndexPath *)indexPath
{
    [self replaceObjectAtIndexPath:indexPath withObject:object];
}

- (void)replaceObjectAtIndexPath:(NSIndexPath *)indexPath withObject:(id)object
{
    [self.objects replaceObjectAtIndex:indexPath.row withObject:object];
}

- (void)removeObjectAtIndexPath:(NSIndexPath *)removeIndexPath andInsertObject:(id)object atIndexPath:(NSIndexPath *)insertIndexPath
{
    [self.objects removeObjectAtIndex:removeIndexPath.row];
    [self.objects insertObject:object atIndex:insertIndexPath.row];
}

- (void)removeObject:(id)object
{
    if ([self.objects containsObject:object]) {
        NSIndexPath *indexPath = [self indexPathOfObject:object];
        [self.tableView beginUpdates];
        [self.tableView deleteRowsAtIndexPaths:@[ indexPath ] withRowAnimation:UITableViewRowAnimationLeft];
        [self.objects removeObject:object];
        [self.tableView endUpdates];
        [self.tableView reloadData];
    }
}

- (void)addObject:(id)object atIndexPath:(NSIndexPath *)indexPath
{
    [self.tableView beginUpdates];
    [self.tableView insertRowsAtIndexPaths:@[ indexPath ] withRowAnimation:UITableViewRowAnimationAutomatic];
    [self.objects insertObject:object atIndex:indexPath.row];
    [self.tableView endUpdates];
    [self.tableView reloadData];
}

- (id)objectAtIndexPath:(NSIndexPath *)indexPath
{
    return [self.objects objectAtIndex:indexPath.row];
}

- (NSIndexPath *)indexPathOfObject:(id)object
{
    return [NSIndexPath indexPathForRow:[self.objects indexOfObject:object] inSection:0];
}

- (id)dummyObject
{
    return @"DUMMY";
}

- (void)configureDefaultSettingsForCell:(UITableViewCell *)cell atIndexPath:(NSIndexPath *)indexPath
{
    cell.contentView.backgroundColor = UIColor.clearColor;
    cell.accessoryType = UITableViewCellAccessoryNone;
}

- (void)configureDummyCell:(UITableViewCell *)cell forIndexPath:(NSIndexPath *)indexPath
{
    cell.textLabel.text = @"";
}

- (void)configureCell:(UITableViewCell *)cell forIndexPath:(NSIndexPath *)indexPath
{

}

- (void)setupTableView
{
    self.tableView = self.reorderableTableView;
    [self.tableView registerClass:self.cellClass forCellReuseIdentifier:self.cellReuseID];
    if (self.headerClass) {
        [self.tableView registerClass:self.headerClass forHeaderFooterViewReuseIdentifier:self.headerReuseID];
    }
}

- (void)viewDidLoad
{
    [super viewDidLoad];
    [self setupTableView];
}

#pragma mark - Table view data source

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView
{
    // TODO: refactor to handle multiple sections
    return 1;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
    return self.objects.count;
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
    UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:self.cellReuseID forIndexPath:indexPath];
    
    [self configureDefaultSettingsForCell:cell atIndexPath:indexPath];
    
    if ([[self objectAtIndexPath:indexPath] isEqual:self.dummyObject]) {
        [self configureDummyCell:cell forIndexPath:indexPath];
    } else {
        [self configureCell:cell forIndexPath:indexPath];
    }
    
    return cell;
}

@end
